      module io_base
!-----------------------------------------------------------------------
      use names
      use io_units
!      implicit none
!-----------------------------------------------------------------------------
! Max. number of AA residues
      integer,parameter :: maxres=6000!1200
! Appr. max. number of interaction sites
      integer,parameter :: maxres2=2*maxres
!      parameter (maxres6=6*maxres)
!      parameter (mmaxres2=(maxres2*(maxres2+1)/2))
!-----------------------------------------------------------------------------
! Max. number of S-S bridges
!      integer,parameter :: maxss=20
!-----------------------------------------------------------------------------
! Max. number of derivatives of virtual-bond and side-chain vectors in theta
! or phi.
!      integer,parameter :: maxdim=(maxres-1)*(maxres-2)/2
!-----------------------------------------------------------------------------
!
!
!-----------------------------------------------------------------------------
      contains
!-----------------------------------------------------------------------------
! readrtns_CSA.F
!-----------------------------------------------------------------------------
      subroutine read_bridge
! Read information about disulfide bridges.
      use geometry_data, only: nres
      use energy_data
      use control_data, only:out1file
      use MPI_data
!      implicit real*8 (a-h,o-z)
!      include 'DIMENSIONS'
#ifdef MPI
      include 'mpif.h'
#endif
!      include 'COMMON.IOUNITS'
!      include 'COMMON.GEO'
!      include 'COMMON.VAR'
!      include 'COMMON.INTERACT'
!      include 'COMMON.LOCAL'
!      include 'COMMON.NAMES'
!      include 'COMMON.CHAIN'
!      include 'COMMON.FFIELD'
!      include 'COMMON.SBRIDGE'
!      include 'COMMON.HEADER'
!      include 'COMMON.CONTROL'
!      include 'COMMON.DBASE'
!      include 'COMMON.THREAD'
!      include 'COMMON.TIME1'
!      include 'COMMON.SETUP'
!el local variables
      integer :: i,j,ierror
      print *,"ENTER READ"
! Read bridging residues.
      read (inp,*) ns
      write(iout,*) "ns",ns
      call flush(iout)
      if (ns.gt.0) then
        allocate(iss(ns))
        read (inp,*) (iss(i),i=1,ns)
      endif

!      print *,'ns=',ns
      if(me.eq.king.or..not.out1file) &
        write (iout,*) 'ns=',ns
      if (ns.gt.0) &
        write(iout,*) ' iss:',(iss(i),i=1,ns)
! Check whether the specified bridging residues are cystines.
      do i=1,ns
         write(iout,*) i,iss(i)
	if (itype(iss(i),1).ne.1) then
	  if (me.eq.king.or..not.out1file) write (iout,'(2a,i3,a)') &
         'Do you REALLY think that the residue ',&
          restyp(itype(iss(i),1),1),i,&
         ' can form a disulfide bridge?!!!'
	  write (*,'(2a,i3,a)') &
         'Do you REALLY think that the residue ',&
          restyp(itype(iss(i),1),1),i,&
         ' can form a disulfide bridge?!!!'
#ifdef MPI
	 call MPI_Finalize(MPI_COMM_WORLD,ierror)
         stop
#endif
        endif
      enddo
      if (dyn_ss) then 
        if(.not.allocated(ihpb)) allocate(ihpb(ns))
        if(.not.allocated(jhpb)) allocate(jhpb(ns))
      endif
! Read preformed bridges.
      if (ns.gt.0) then
        read (inp,*) nss
      if (nss.gt.0) then
        if(.not.allocated(ihpb)) allocate(ihpb(nss))
        if(.not.allocated(jhpb)) allocate(jhpb(nss))
        read (inp,*) (ihpb(i),jhpb(i),i=1,nss)

        if(.not.allocated(dhpb)) allocate(dhpb(nss))
        if(.not.allocated(forcon)) allocate(forcon(nss))!(maxdim) !el maxdim=(maxres-1)*(maxres-2)/2
        if(.not.allocated(dhpb1)) allocate(dhpb1(nss))
        if(.not.allocated(ibecarb)) allocate(ibecarb(nss))
! el Initialize the bridge array
        do i=1,nss
	  dhpb(i)=0.0D0
        enddo
      endif
!--------------------
      if(fg_rank.eq.0) &
       write(iout,*)'nss=',nss !el,' ihpb,jhpb: ',(ihpb(i),jhpb(i),i=1,nss)
      if (nss.gt.0) then
       write(iout,*)'ihpb,jhpb: ',(ihpb(i),jhpb(i),i=1,nss)
        nhpb=nss
! Check if the residues involved in bridges are in the specified list of
! bridging residues.
        do i=1,nss
          do j=1,i-1
	    if (ihpb(i).eq.ihpb(j).or.ihpb(i).eq.jhpb(j) &
            .or.jhpb(i).eq.ihpb(j).or.jhpb(i).eq.jhpb(j)) then
	      write (iout,'(a,i3,a)') 'Disulfide pair',i,&
            ' contains residues present in other pairs.'
	      write (*,'(a,i3,a)') 'Disulfide pair',i,&
            ' contains residues present in other pairs.'
#ifdef MPI
	      call MPI_Finalize(MPI_COMM_WORLD,ierror)
              stop 
#endif
	    endif
          enddo
	  do j=1,ns
	    if (ihpb(i).eq.iss(j)) goto 10
          enddo
          write (iout,'(a,i3,a)') 'Pair',i,' contains unknown cystine.'
   10     continue
	  do j=1,ns
	    if (jhpb(i).eq.iss(j)) goto 20
          enddo
          write (iout,'(a,i3,a)') 'Pair',i,' contains unknown cystine.'
   20     continue
          dhpb(i)=d0cm
          forcon(i)=akcm
        enddo
        do i=1,nss
          ihpb(i)=ihpb(i)+nres
          jhpb(i)=jhpb(i)+nres
        enddo
      endif
      endif
      write(iout,*) "end read_bridge"
      return
      end subroutine read_bridge
!-----------------------------------------------------------------------------
!      subroutine read_x(kanal,*)
!
!     use geometry_data
!     use energy_data
!     use geometry, only:int_from_cart1
!!      implicit real*8 (a-h,o-z)
!!      include 'DIMENSIONS'
!!      include 'COMMON.GEO'
!!      include 'COMMON.VAR'
!!      include 'COMMON.CHAIN'
!!      include 'COMMON.IOUNITS'
!!      include 'COMMON.CONTROL'
!      include 'COMMON.LOCAL'
!      include 'COMMON.INTERACT'
! Read coordinates from input
!
!el local variables
!      integer :: l,k,j,i,kanal
!
!      read(kanal,'(8f10.5)',end=10,err=10) &
!        ((c(l,k),l=1,3),k=1,nres),&
!        ((c(l,k+nres),l=1,3),k=nnt,nct)
!      do j=1,3
!        c(j,nres+1)=c(j,1)
!        c(j,2*nres)=c(j,nres)
!      enddo
!      call int_from_cart1(.false.)
!      do i=1,nres-1
!        do j=1,3
!          dc(j,i)=c(j,i+1)-c(j,i)
!          dc_norm(j,i)=dc(j,i)*vbld_inv(i+1)
!        enddo
!      enddo
!      do i=nnt,nct
!!        if (itype(i,1).ne.10 .and. itype(i,1).ne.ntyp1) then
!!          do j=1,3
!!            dc(j,i+nres)=c(j,i+nres)-c(j,i)
!!            dc_norm(j,i+nres)=dc(j,i+nres)*vbld_inv(i+nres)
!!          enddo
!!        endif
!!      enddo
!
!      return
!   10 return 1
!      end subroutine read_x
!!-----------------------------------------------------------------------------
      subroutine read_threadbase

      use geometry_data
      use energy_data
      use compare_data
!      implicit real*8 (a-h,o-z)
!      include 'DIMENSIONS'
!      include 'COMMON.IOUNITS'
!      include 'COMMON.GEO'
!      include 'COMMON.VAR'
!      include 'COMMON.INTERACT'
!      include 'COMMON.LOCAL'
!      include 'COMMON.NAMES'
!      include 'COMMON.CHAIN'
!      include 'COMMON.FFIELD'
!      include 'COMMON.SBRIDGE'
!      include 'COMMON.HEADER'
!      include 'COMMON.CONTROL'
!      include 'COMMON.DBASE'
!      include 'COMMON.THREAD'
!      include 'COMMON.TIME1'

!el local variables
      integer :: k,j,i

! Read pattern database for threading.
      read (icbase,*) nseq
      allocate(cart_base(3,maxres_base,nseq)) !(3,maxres_base,maxseq)
      allocate(nres_base(3,nseq)) !(3,maxseq)
      allocate(str_nam(nseq)) !(maxseq)
      do i=1,nseq
        read (icbase,'(i5,2x,a8,2i4)') nres_base(1,i),str_nam(i),&
         nres_base(2,i),nres_base(3,i)
        read (icbase,'(9f8.3)') ((cart_base(k,j,i),k=1,3),j=1,&
         nres_base(1,i))
!       write (iout,'(i5,2x,a8,2i4)') nres_base(1,i),str_nam(i),
!    &   nres_base(2,i),nres_base(3,i)
!       write (iout,'(9f8.3)') ((cart_base(k,j,i),k=1,3),j=1,
!    &   nres_base(1,i))
      enddo
      close (icbase)
      if (weidis.eq.0.0D0) weidis=0.1D0
      do i=nnt,nct
        do j=i+2,nct
          nhpb=nhpb+1
          ihpb(nhpb)=i
          jhpb(nhpb)=j
          forcon(nhpb)=weidis
        enddo
      enddo 
      read (inp,*) nexcl,(iexam(1,i),iexam(2,i),i=1,nexcl)
      write (iout,'(a,i5)') 'nexcl: ',nexcl
      write (iout,'(2i5)') (iexam(1,i),iexam(2,i),i=1,nexcl)
      return
      end subroutine read_threadbase
!-----------------------------------------------------------------------------
#ifdef WHAM_RUN
!el      subroutine read_angles(kanal,iscor,energ,iprot,*)
      subroutine read_angles(kanal,*)

      use geometry_data
      use energy_data
!      implicit real*8 (a-h,o-z)
!      include 'DIMENSIONS'
!      include 'DIMENSIONS.ZSCOPT'
!      include 'COMMON.INTERACT'
!      include 'COMMON.SBRIDGE'
!      include 'COMMON.GEO'
!      include 'COMMON.VAR'
!      include 'COMMON.CHAIN'
!      include 'COMMON.IOUNITS'
      character(len=80) :: lineh
      integer :: iscor,iprot,ic
      real(kind=8) :: energ
#else
      subroutine read_angles(kanal,*)

      use geometry_data
   !  use energy
   !  use control
!      implicit real*8 (a-h,o-z)
!      include 'DIMENSIONS'
!      include 'COMMON.GEO'
!      include 'COMMON.VAR'
!      include 'COMMON.CHAIN'
!      include 'COMMON.IOUNITS'
!      include 'COMMON.CONTROL'
#endif
! Read angles from input 
!
!el local variables
      integer :: i,kanal
#ifdef WHAM_RUN
      read(kanal,'(a80)',end=10,err=10) lineh
      read(lineh(:5),*,err=8) ic
      read(lineh(6:),*,err=8) energ
      goto 9
    8 ic=1
      print *,'error, assuming e=1d10',lineh
      energ=1d10
      nss=0
    9 continue
      read(lineh(18:),*,end=10,err=10) nss
      IF (NSS.LT.9) THEN
        read (lineh(20:),*,end=10,err=10) &
        (IHPB(I),JHPB(I),I=1,NSS),iscor
      ELSE
        read (lineh(20:),*,end=10,err=10) (IHPB(I),JHPB(I),I=1,8)
        read (kanal,*,end=10,err=10) (IHPB(I),JHPB(I),&
          I=9,NSS),iscor
      ENDIF
!      print *,"energy",energ," iscor",iscor
#endif
       read (kanal,*,err=10,end=10) (theta(i),i=3,nres)
       read (kanal,*,err=10,end=10) (phi(i),i=4,nres)
       read (kanal,*,err=10,end=10) (alph(i),i=2,nres-1)
       read (kanal,*,err=10,end=10) (omeg(i),i=2,nres-1)

       do i=1,nres
! 9/7/01 avoid 180 deg valence angle
        if (theta(i).gt.179.99d0) theta(i)=179.99d0
!
        theta(i)=deg2rad*theta(i)
        phi(i)=deg2rad*phi(i)
        alph(i)=deg2rad*alph(i)
        omeg(i)=deg2rad*omeg(i)
       enddo
      return
   10 return 1
      end subroutine read_angles
!-----------------------------------------------------------------------------
      subroutine reada(rekord,lancuch,wartosc,default)

!      implicit none
      character*(*) :: rekord,lancuch
      real(kind=8) :: wartosc,default
      integer :: iread !,ilen
!el      external ilen
      iread=index(rekord,lancuch)
      if (iread.eq.0) then
        wartosc=default 
        return
      endif   
      iread=iread+ilen(lancuch)+1
      read (rekord(iread:),*,err=10,end=10) wartosc
      return
  10  wartosc=default
      return
      end subroutine reada
!-----------------------------------------------------------------------------
      subroutine readi(rekord,lancuch,wartosc,default)

!      implicit none
      character*(*) :: rekord,lancuch
      integer :: wartosc,default
      integer :: iread !,ilen
!el      external ilen
      iread=index(rekord,lancuch)
      if (iread.eq.0) then
        wartosc=default 
        return
      endif   
      iread=iread+ilen(lancuch)+1
      read (rekord(iread:),*,err=10,end=10) wartosc
      return
  10  wartosc=default
      return
      end subroutine readi
!-----------------------------------------------------------------------------
      subroutine multreadi(rekord,lancuch,tablica,dim,default)

!      implicit none
      integer :: dim,i
      integer :: tablica(dim),default
      character*(*) :: rekord,lancuch
      character(len=80) :: aux
      integer :: iread !,ilen
!el      external ilen
      do i=1,dim
        tablica(i)=default
      enddo
      iread=index(rekord,lancuch(:ilen(lancuch))//"=")
      if (iread.eq.0) return
      iread=iread+ilen(lancuch)+1
      read (rekord(iread:),*,end=10,err=10) (tablica(i),i=1,dim)
   10 return
      end subroutine multreadi
!-----------------------------------------------------------------------------
      subroutine multreada(rekord,lancuch,tablica,dim,default)

!      implicit none
      integer :: dim,i
      real(kind=8) :: tablica(dim),default
      character*(*) :: rekord,lancuch
      character(len=80) :: aux
      integer :: iread !,ilen
!el      external ilen
      do i=1,dim
        tablica(i)=default
      enddo
      iread=index(rekord,lancuch(:ilen(lancuch))//"=")
      if (iread.eq.0) return
      iread=iread+ilen(lancuch)+1
      read (rekord(iread:),*,end=10,err=10) (tablica(i),i=1,dim)
   10 return
      end subroutine multreada
!-----------------------------------------------------------------------------
      subroutine card_concat(card,to_upper)

! dla UNRESA to_upper jest zawsze .true.
!      implicit real*8 (a-h,o-z)
!      include 'DIMENSIONS'
!      include 'COMMON.IOUNITS'
      character(*) :: card
      character(len=80) :: karta	!,ucase
      logical :: to_upper
!el      external ilen
      read (inp,'(a)') karta
      if (to_upper) karta=ucase(karta)
      card=' '
      do while (karta(80:80).eq.'&')
        card=card(:ilen(card)+1)//karta(:79)
        read (inp,'(a)') karta
        if (to_upper) karta=ucase(karta)
      enddo
      card=card(:ilen(card)+1)//karta
      return
      end subroutine card_concat
!----------------------------------------------------------------------------
      subroutine read_afminp
      use geometry_data
      use energy_data
      use control_data, only:out1file
      use MPI_data
      character*320 afmcard
      integer i
      print *, "wchodze"
      call card_concat(afmcard,.true.)
      call readi(afmcard,"BEG",afmbeg,0)
      call readi(afmcard,"END",afmend,0)
      call reada(afmcard,"FORCE",forceAFMconst,0.0d0)
      call reada(afmcard,"VEL",velAFMconst,0.0d0)
      print *,'FORCE=' ,forceAFMconst
!------ NOW PROPERTIES FOR AFM
       distafminit=0.0d0
       do i=1,3
        distafminit=(c(i,afmend)-c(i,afmbeg))**2+distafminit
       enddo
        distafminit=dsqrt(distafminit)
        print *,'initdist',distafminit
      return
      end subroutine read_afminp
!-----------------------------------------------------------------------------
!      subroutine read_dist_constr
!      use MPI_data
!   !  use control
!      use geometry, only: dist
!      use geometry_data
!      use control_data
!      use energy_data
!!      implicit real*8 (a-h,o-z)
!!      include 'DIMENSIONS'
!!#ifdef MPI
!      include 'mpif.h'
!#endif
!      include 'COMMON.SETUP'
!      include 'COMMON.CONTROL'
!      include 'COMMON.CHAIN'
!      include 'COMMON.IOUNITS'
!      include 'COMMON.SBRIDGE'
!      integer,dimension(2,100) :: ifrag_,ipair_
!      real(kind=8),dimension(100) :: wfrag_,wpair_
!      character(len=640) :: controlcard
!
!!el local variables
!      integer :: i,k,j,ddjk,ii,jj,itemp
!      integer :: nfrag_,npair_,ndist_
!      real(kind=8) :: dist_cut
!
!!      write (iout,*) "Calling read_dist_constr"
!      write (iout,*) "nres",nres," nstart_sup",nstart_sup," nsup",nsup
!      call flush(iout)
!      if(.not.allocated(ihpb)) allocate(ihpb(maxdim))
!      if(.not.allocated(jhpb)) allocate(jhpb(maxdim))
!      if(.not.allocated(dhpb)) allocate(dhpb(maxdim))
!      if(.not.allocated(dhpb1)) allocate(dhpb1(maxdim))
!      if(.not.allocated(forcon)) allocate(forcon(maxdim))
!      if(.not.allocated(fordepth)) allocate(fordepth(maxdim))
!      if ((genconstr.gt.0).and.(constr_dist.eq.11)) then
!      call gen_dist_constr2
!      go to 1712
!      endif
!      call card_concat(controlcard,.true.)
!      call readi(controlcard,"NFRAG",nfrag_,0)
!      call readi(controlcard,"NPAIR",npair_,0)
!      call readi(controlcard,"NDIST",ndist_,0)
!      call reada(controlcard,'DIST_CUT',dist_cut,5.0d0)
!      call multreadi(controlcard,"IFRAG",ifrag_(1,1),2*nfrag_,0)
!      call multreadi(controlcard,"IPAIR",ipair_(1,1),2*npair_,0)
!      call multreada(controlcard,"WFRAG",wfrag_(1),nfrag_,0.0d0)
!      call multreada(controlcard,"WPAIR",wpair_(1),npair_,0.0d0)
!!      write (iout,*) "NFRAG",nfrag_," NPAIR",npair_," NDIST",ndist_
!      write (iout,*) "IFRAG"
!      do i=1,nfrag_
!        write (iout,*) i,ifrag_(1,i),ifrag_(2,i),wfrag_(i)
!      enddo
!      write (iout,*) "IPAIR"
!      do i=1,npair_
!        write (iout,*) i,ipair_(1,i),ipair_(2,i),wpair_(i)
!      enddo
!      if(.not.allocated(ihpb)) allocate(ihpb(maxdim))
!      if(.not.allocated(jhpb)) allocate(jhpb(maxdim))
!      if(.not.allocated(dhpb)) allocate(dhpb(maxdim))
!      if(.not.allocated(forcon)) allocate(forcon(maxdim))
!      
!      call flush(iout)
!      do i=1,nfrag_
!        if (ifrag_(1,i).lt.nstart_sup) ifrag_(1,i)=nstart_sup
!        if (ifrag_(2,i).gt.nstart_sup+nsup-1) &
!          ifrag_(2,i)=nstart_sup+nsup-1
!!        write (iout,*) i,ifrag_(1,i),ifrag_(2,i),wfrag_(i)
!        call flush(iout)
!        if (wfrag_(i).gt.0.0d0) then
!        do j=ifrag_(1,i),ifrag_(2,i)-1
!          do k=j+1,ifrag_(2,i)
!!            write (iout,*) "j",j," k",k
!            ddjk=dist(j,k)
!            if (constr_dist.eq.1) then
!              nhpb=nhpb+1
!              ihpb(nhpb)=j
!              jhpb(nhpb)=k
!              dhpb(nhpb)=ddjk
!              forcon(nhpb)=wfrag_(i) 
!            else if (constr_dist.eq.2) then
!              if (ddjk.le.dist_cut) then
!                nhpb=nhpb+1
!                ihpb(nhpb)=j
!                jhpb(nhpb)=k
!                dhpb(nhpb)=ddjk
!                forcon(nhpb)=wfrag_(i) 
!              endif
!            else
!              nhpb=nhpb+1
!              ihpb(nhpb)=j
!              jhpb(nhpb)=k
!              dhpb(nhpb)=ddjk
!              forcon(nhpb)=wfrag_(i)*dexp(-0.5d0*(ddjk/dist_cut)**2)
!            endif
!#ifdef MPI
!            if (.not.out1file .or. me.eq.king) &
!            write (iout,'(a,3i5,f8.2,1pe12.2)') "+dist.constr ",&
!             nhpb,ihpb(nhpb),jhpb(nhpb),dhpb(nhpb),forcon(nhpb)
!#else
!            write (iout,'(a,3i5,f8.2,1pe12.2)') "+dist.constr ",&
!             nhpb,ihpb(nhpb),jhpb(nhpb),dhpb(nhpb),forcon(nhpb)
!#endif
!          enddo
!        enddo
!        endif
!      enddo
!      do i=1,npair_
!        if (wpair_(i).gt.0.0d0) then
!        ii = ipair_(1,i)
!        jj = ipair_(2,i)
!        if (ii.gt.jj) then
!          itemp=ii
!          ii=jj
!          jj=itemp
!        endif
!        do j=ifrag_(1,ii),ifrag_(2,ii)
!          do k=ifrag_(1,jj),ifrag_(2,jj)
!            nhpb=nhpb+1
!            ihpb(nhpb)=j
!            jhpb(nhpb)=k
!            forcon(nhpb)=wpair_(i)
!            dhpb(nhpb)=dist(j,k)
!#ifdef MPI
!            if (.not.out1file .or. me.eq.king) &
!            write (iout,'(a,3i5,f8.2,f10.1)') "+dist.constr ",&
!             nhpb,ihpb(nhpb),jhpb(nhpb),dhpb(nhpb),forcon(nhpb)
!#else
!            write (iout,'(a,3i5,f8.2,f10.1)') "+dist.constr ",&
!             nhpb,ihpb(nhpb),jhpb(nhpb),dhpb(nhpb),forcon(nhpb)
!#endif
!          enddo
!        enddo
!        endif
!      enddo 
!      do i=1,ndist_
!!        read (inp,*) ihpb(nhpb+1),jhpb(nhpb+1),forcon(nhpb+1)
!!        if (forcon(nhpb+1).gt.0.0d0) then
!!          nhpb=nhpb+1
!!          dhpb(nhpb)=dist(ihpb(nhpb),jhpb(nhpb))
!        if (constr_dist.eq.11) then
!        read (inp,*) ihpb(nhpb+1),jhpb(nhpb+1),dhpb(i),dhpb1(i), &
!          ibecarb(i),forcon(nhpb+1),fordepth(nhpb+1)
!        fordepth(nhpb+1)=fordepth(nhpb+1)/forcon(nhpb+1)
!        else
!!C        print *,"in else"
!        read (inp,*) ihpb(nhpb+1),jhpb(nhpb+1),dhpb(i),dhpb1(i), &
!          ibecarb(i),forcon(nhpb+1)
!        endif
!        if (forcon(nhpb+1).gt.0.0d0) then
!          nhpb=nhpb+1
!          if (ibecarb(i).gt.0) then
!            ihpb(i)=ihpb(i)+nres
!            jhpb(i)=jhpb(i)+nres
!          endif
!          if (dhpb(nhpb).eq.0.0d0) &
!            dhpb(nhpb)=dist(ihpb(nhpb),jhpb(nhpb))
!        endif
!
!#ifdef MPI
!          if (.not.out1file .or. me.eq.king) &
!          write (iout,'(a,3i5,f8.2,f10.1)') "+dist.constr ",&
!           nhpb,ihpb(nhpb),jhpb(nhpb),dhpb(nhpb),forcon(nhpb)
!#else
!          write (iout,'(a,3i5,f8.2,f10.1)') "+dist.constr ",&
!           nhpb,ihpb(nhpb),jhpb(nhpb),dhpb(nhpb),forcon(nhpb)
!#endif
!      enddo
! 1712 continue
!      call flush(iout)
!      return
!      end subroutine read_dist_constr
!-----------------------------------------------------------------------------
!      subroutine gen_dist_constr2
!      use MPI_data
!   !  use control
!      use geometry, only: dist
!      use geometry_data
!      use control_data
!      use energy_data
!      integer :: i,j
!      real(kind=8) :: distance
!      if (constr_dist.eq.11) then
!             do i=nstart_sup,nstart_sup+nsup-1
!              do j=i+2,nstart_sup+nsup-1
!                 distance=dist(i,j)
!                 if (distance.le.15.0) then
!                 nhpb=nhpb+1
!                 ihpb(nhpb)=i+nstart_seq-nstart_sup
!                 jhpb(nhpb)=j+nstart_seq-nstart_sup
!                 forcon(nhpb)=sqrt(0.04*distance)
!                 fordepth(nhpb)=sqrt(40.0/distance)
!                 dhpb(nhpb)=distance-0.1d0
!                 dhpb1(nhpb)=distance+0.1d0
!
!#ifdef MPI
!          if (.not.out1file .or. me.eq.king) &
!          write (iout,'(a,3i5,f8.2,f10.2)') "+dist.constr ", &
!          nhpb,ihpb(nhpb),jhpb(nhpb),dhpb(nhpb),forcon(nhpb)
!#else
!          write (iout,'(a,3i5,f8.2,f10.2)') "+dist.constr ", &
!          nhpb,ihpb(nhpb),jhpb(nhpb),dhpb(nhpb),forcon(nhpb)
!#endif
!            endif
!             enddo
!           enddo
!      else
!      do i=nstart_sup,nstart_sup+nsup-1
!        do j=i+2,nstart_sup+nsup-1
!          nhpb=nhpb+1
!          ihpb(nhpb)=i+nstart_seq-nstart_sup
!          jhpb(nhpb)=j+nstart_seq-nstart_sup
!          forcon(nhpb)=weidis
!          dhpb(nhpb)=dist(i,j)
!        enddo
!      enddo
!      endif
!      return
!      end subroutine gen_dist_constr2
!
!!-----------------------------------------------------------------------------
#ifdef WINIFL
      subroutine flush(iu)
      return
      end subroutine flush
#endif
#ifdef AIX
      subroutine flush(iu)
      call flush_(iu)
      return
      end subroutine flush
#endif
!-----------------------------------------------------------------------------
      subroutine copy_to_tmp(source)

!      include "DIMENSIONS"
!      include "COMMON.IOUNITS"
      character*(*) :: source
      character(len=256) :: tmpfile
!      integer ilen
!el      external ilen
      logical :: ex
      tmpfile=curdir(:ilen(curdir))//"/"//source(:ilen(source))
      inquire(file=tmpfile,exist=ex)
      if (ex) then
        write (*,*) "Copying ",tmpfile(:ilen(tmpfile)),&
         " to temporary directory..."
        write (*,*) "/bin/cp "//tmpfile(:ilen(tmpfile))//" "//tmpdir
        call system("/bin/cp "//tmpfile(:ilen(tmpfile))//" "//tmpdir)
      endif
      return
      end subroutine copy_to_tmp
!-----------------------------------------------------------------------------
      subroutine move_from_tmp(source)

!      include "DIMENSIONS"
!      include "COMMON.IOUNITS"
      character*(*) :: source
!      integer ilen
!el      external ilen
      write (*,*) "Moving ",source(:ilen(source)),&
       " from temporary directory to working directory"
      write (*,*) "/bin/mv "//source(:ilen(source))//" "//curdir
      call system("/bin/mv "//source(:ilen(source))//" "//curdir)
      return
      end subroutine move_from_tmp
!-----------------------------------------------------------------------------
! misc.f
!-----------------------------------------------------------------------------
! $Date: 1994/10/12 17:24:21 $
! $Revision: 2.5 $

      logical function find_arg(ipos,line,errflag)

      integer, parameter :: maxlen = 80
      character(len=80) :: line
      character(len=1) :: empty=' ',equal='='
      logical :: errflag
      integer :: ipos
! This function returns .TRUE., if an argument follows keyword keywd; if so
! IPOS will point to the first non-blank character of the argument. Returns
! .FALSE., if no argument follows the keyword; in this case IPOS points
! to the first non-blank character of the next keyword.

      do while (line(ipos:ipos) .eq. empty .and. ipos.le.maxlen)
        ipos=ipos+1
      enddo 
      errflag=.false.
      if (line(ipos:ipos).eq.equal) then
         find_arg=.true.
         ipos=ipos+1
         do while (line(ipos:ipos) .eq. empty .and. ipos.le.maxlen)
           ipos=ipos+1
         enddo
         if (ipos.gt.maxlen) errflag=.true.
      else
         find_arg=.false.
      endif

      return
      end function find_arg
!-----------------------------------------------------------------------------
      logical function find_group(iunit,jout,key1)

      character*(*) :: key1
      character(len=80) :: karta	!,ucase
      integer :: iunit,jout
      integer :: ll !,ilen
!EL      external ilen
!EL      logical lcom
      rewind (iunit)
      karta=' '
      ll=ilen(key1)
      do while (index(ucase(karta),key1(1:ll)).eq.0.or.lcom(1,karta)) 
        read (iunit,'(a)',end=10) karta
      enddo
      write (jout,'(2a)') '> ',karta(1:78)
      find_group=.true.
      return
   10 find_group=.false.
      return
      end function find_group
!-----------------------------------------------------------------------------
      logical function iblnk(charc)
      character(len=1) :: charc
      integer :: n
      n = ichar(charc)
      iblnk = (n.eq.9) .or. (n.eq.10) .or. (charc.eq.' ')
      return
      end function iblnk
!-----------------------------------------------------------------------------
      integer function ilen(string)
      character*(*) ::  string
!EL      logical :: iblnk

      ilen = len(string)
1     if ( ilen .gt. 0 ) then
         if ( iblnk( string(ilen:ilen) ) ) then
            ilen = ilen - 1
            goto 1
         endif
      endif
      return
      end function ilen
!-----------------------------------------------------------------------------
      integer function in_keywd_set(nkey,ikey,narg,keywd,keywdset)
      integer :: nkey,i,ikey,narg
      character(len=16) :: keywd,keywdset(1:nkey,0:nkey)
!      character(len=16) :: ucase

      do i=1,narg
        if (ucase(keywd).eq.keywdset(i,ikey)) then
! Match found
          in_keywd_set=i
          return
        endif
      enddo
! No match to the allowed set of keywords if this point is reached. 
      in_keywd_set=0
      return
      end function in_keywd_set
!-----------------------------------------------------------------------------
      character function lcase(string)
      integer :: i, k, idiff
      character*(*) :: string
      character(len=1) :: c
      character(len=40) :: chtmp
!
      i = len(lcase)
      k = len(string)
      if (i .lt. k) then
         k = i
         if (string(k+1:) .ne. ' ') then
            chtmp = string
         endif
      endif
      idiff = ichar('a') - ichar('A')
      lcase = string
      do 99 i = 1, k
         c = string(i:i)
         if (lge(c,'A') .and. lle(c,'Z')) then
            lcase(i:i) = char(ichar(c) + idiff)
         endif
   99 continue
      return
      end function lcase
!-----------------------------------------------------------------------------
      logical function lcom(ipos,karta)
      character(len=80) :: karta
      character :: koment(2) = (/'!','#'/)
      integer :: ipos,i

      lcom=.false.
      do i=1,2
        if (karta(ipos:ipos).eq.koment(i)) lcom=.true.
      enddo
      return
      end function lcom
!-----------------------------------------------------------------------------
      logical function lower_case(ch)
      character*(*) :: ch
      lower_case=(ch.ge.'a' .and. ch.le.'z')
      return
      end function lower_case
!-----------------------------------------------------------------------------
      subroutine mykey(line,keywd,ipos,blankline,errflag)

! This subroutine seeks a non-empty substring keywd in the string LINE.
! The substring begins with the first character different from blank and
! "=" encountered right to the pointer IPOS (inclusively) and terminates
! at the character left to the first blank or "=". When the subroutine is 
! exited, the pointer IPOS is moved to the position of the terminator in LINE. 
! The logical variable BLANKLINE is set at .TRUE., if LINE(IPOS:) contains
! only separators or the maximum length of the data line (80) has been reached.
! The logical variable ERRFLAG is set at .TRUE. if the string 
! consists only from a "=".
      integer, parameter :: maxlen=80
      character(len=1) :: empty=' ',equal='=',comma=','
      character*(*) :: keywd
      character(len=80) :: line
      logical :: blankline,errflag	!EL,lcom
      integer :: ipos,istart,iend
      errflag=.false.
      do while (line(ipos:ipos).eq.empty .and. (ipos.le.maxlen))
        ipos=ipos+1
      enddo
      if (ipos.gt.maxlen .or. lcom(ipos,line) ) then
! At this point the rest of the input line turned out to contain only blanks
! or to be commented out.
        blankline=.true.
        return
      endif
      blankline=.false.
      istart=ipos
! Checks whether the current char is a separator.
      do while (line(ipos:ipos).ne.empty .and. line(ipos:ipos).ne.equal &
       .and. line(ipos:ipos).ne.comma .and. ipos.le.maxlen)
        ipos=ipos+1
      enddo
      iend=ipos-1
! Error flag set to .true., if the length of the keyword was found less than 1.
      if (iend.lt.istart) then
        errflag=.true.
        return
      endif
      keywd=line(istart:iend)
      return
      end subroutine  mykey
!-----------------------------------------------------------------------------
      subroutine numstr(inum,numm)
      character(len=10) :: huj='0123456789'
      character*(*) :: numm
      integer :: inumm,inum,inum1,inum2
      inumm=inum
      inum1=inumm/10
      inum2=inumm-10*inum1
      inumm=inum1
      numm(3:3)=huj(inum2+1:inum2+1)
      inum1=inumm/10
      inum2=inumm-10*inum1
      inumm=inum1
      numm(2:2)=huj(inum2+1:inum2+1)
      inum1=inumm/10
      inum2=inumm-10*inum1 
      inumm=inum1
      numm(1:1)=huj(inum2+1:inum2+1)
      return
      end subroutine numstr
!-----------------------------------------------------------------------------
      function ucase(string)
      integer :: i, k, idiff
      character(*) :: string
      character(len=len(string)) :: ucase
      character(len=1) :: c
      character(len=40) :: chtmp
!
      i = len(ucase)
      k = len(string)
      if (i .lt. k) then
         k = i
         if (string(k+1:) .ne. ' ') then
            chtmp = string
         endif
      endif
      idiff = ichar('a') - ichar('A')
      ucase = string
      do 99 i = 1, k
         c = string(i:i)
         if (lge(c,'a') .and. lle(c,'z')) then
            ucase(i:i) = char(ichar(c) - idiff)
         endif
   99 continue
      return
      end function ucase
!-----------------------------------------------------------------------------
! geomout.F
!-----------------------------------------------------------------------------
      subroutine pdbout(tytul,iunit)

      use geometry_data
      use energy_data
   !  use control
      use compare_data
      use MD_data
      use names
!      implicit real*8 (a-h,o-z)
!      include 'DIMENSIONS'
!      include 'COMMON.CHAIN'
!      include 'COMMON.INTERACT'
!      include 'COMMON.NAMES'
!      include 'COMMON.IOUNITS'
!      include 'COMMON.HEADER'
!      include 'COMMON.SBRIDGE'
!      include 'COMMON.DISTFIT'
!      include 'COMMON.MD'
!el      character(len=50) :: tytul
      character*(*) :: tytul
      character(len=1),dimension(10) :: chainid= (/'A','B','C','D','E', &
      'F','G','H','I','J'/)
      integer,dimension(nres) :: ica	!(maxres)
      integer iti1
!el  local variables
      integer :: j,iti,itj,itk,itl,i,iatom,ichain,ires,iunit
!      real(kind=8) :: etot
!      integer :: nres2
!      nres2=2*nres

!      if(.not.allocated(vtot)) allocate(vtot(nres2)) !(maxres2)

!      write (iunit,'(3a,1pe15.5)') 'REMARK ',tytul,' ENERGY ',etot
!model      write (iunit,'(a5,i6)') 'MODEL',1
!      if (nhfrag.gt.0) then
!       do j=1,nhfrag
!        iti=itype(hfrag(1,j),1)
!        itj=itype(hfrag(2,j),1)
!        if (j.lt.10) then
!           write (iunit,'(a5,i5,1x,a1,i1,2x,a3,i7,2x,a3,i7,i3,t76,i5)') &
!                 'HELIX',j,'H',j,&
!                 restyp(iti,1),hfrag(1,j)-1,&
!                 restyp(itj,1),hfrag(2,j)-1,1,hfrag(2,j)-hfrag(1,j)
!        else
!             write (iunit,'(a5,i5,1x,a1,i2,1x,a3,i7,2x,a3,i7,i3)') &
!                 'HELIX',j,'H',j,&
!                 restyp(iti,1),hfrag(1,j)-1,&
!                 restyp(itj,1),hfrag(2,j)-1,1,hfrag(2,j)-hfrag(1,j)
!        endif
!       enddo
!      endif
!
!      if (nbfrag.gt.0) then
!
!       do j=1,nbfrag
!
!        iti=itype(bfrag(1,j),1)
!        itj=itype(bfrag(2,j)-1,1)
!
!        write (iunit,'(a5,i5,1x,a1,i1,i3,1x,a3,i6,2x,a3,i6,i3)') &
!                 'SHEET',1,'B',j,2,&
!                 restyp(iti,1),bfrag(1,j)-1,&
!                 restyp(itj,1),bfrag(2,j)-2,0
!
!        if (bfrag(3,j).gt.bfrag(4,j)) then
!
!         itk=itype(bfrag(3,j),1)
!         itl=itype(bfrag(4,j)+1,1)
!
!         write (iunit,'(a5,i5,1x,a1,i1,i3,1x,a3,i6,2x,a3,i6,i3,2x,a1,2x,a3,i6,3x,a1,2x,a3,i6)') &
!                 'SHEET',2,'B',j,2,&
!                 restyp(itl,1),bfrag(4,j),&
!                 restyp(itk,1),bfrag(3,j)-1,-1,&
!                 "N",restyp(itk,1),bfrag(3,j)-1,&
!                 "O",restyp(iti,1),bfrag(1,j)-1
!
!        else
!
!         itk=itype(bfrag(3,j),1)
!         itl=itype(bfrag(4,j)-1,1)
!
!
!        write (iunit,'(a5,i5,1x,a1,i1,i3,1x,a3,i6,2x,a3,i6,i3,2x,a1,2x,a3,i6,3x,a1,2x,a3,i6)') &
!                 'SHEET',2,'B',j,2,&
!                 restyp(itk,1),bfrag(3,j)-1,&
!                 restyp(itl,1),bfrag(4,j)-2,1,&
!                 "N",restyp(itk,1),bfrag(3,j)-1,&
!                 "O",restyp(iti,1),bfrag(1,j)-1
!
!
!
!        endif
!         
!       enddo
!      endif 
!
!      if (nss.gt.0) then
!        do i=1,nss
!         if (dyn_ss) then
!          write(iunit,'(a6,i4,1x,a3,i7,4x,a3,i7)') &
!               'SSBOND',i,'CYS',idssb(i)-nnt+1,&
!                          'CYS',jdssb(i)-nnt+1
!         else
!          write(iunit,'(a6,i4,1x,a3,i7,4x,a3,i7)') &
!               'SSBOND',i,'CYS',ihpb(i)-nnt+1-nres,&
!                          'CYS',jhpb(i)-nnt+1-nres
!         endif
!        enddo
!      endif
!      
      iatom=1
      ichain=1
      ires=0
      do i=1,nres
         iti=itype(i,molnum(i))
         print *,i,molnum(i)
         if (molnum(i+1).eq.0) then
            iti1=ntyp1_molec(molnum(i))
         else
            iti1=itype(i+1,molnum(i+1))
         endif
         if ((iti.eq.ntyp1_molec(molnum(i))).and.(iti1.eq.ntyp1_molec(molnum(i)))) cycle
         if (i.lt.nnt) then
            if (iti.eq.ntyp1_molec(molnum(i)).and.(molnum(i+1).eq.5)) cycle
         endif
         if (iti.eq.ntyp1_molec(molnum(i))) then
            ichain=ichain+1
!          ires=0
            write (iunit,'(a)') 'TER'
         else
            ires=ires+1
!            iatom=iatom+1
            ica(i)=iatom
            if (molnum(i).eq.1) then
               write (iunit,10) iatom,restyp(iti,molnum(i)),chainid(ichain),&
                                        ires,(c(j,i),j=1,3),vtot(i)
!-------------------------------------------------------------------------------
!   tutaj dzieje sie magia
!-------------------------------------------------------------------------------
            elseif(molnum(i).eq.2) then
               if (istype(i).eq.0) istype(i)=1
               if ((iti.ne.ntyp1_molec(2)).and.(molnum(i-1).eq.2).and.(itype(i-1,2).ne.ntyp1_molec(2))) then
                  nat=resnatom(istype(i),iti)+4
               else
                  nat=resnatom(istype(i),iti)
               endif
               print *, i,iti,restyp(iti,2),sugartyp(istype(i)),resnatom(istype(i),iti)
               do j=1,nat
                  write (iunit,40) iatom,atname(iatom),sugartyp(istype(i)),restyp(iti,2), &
                      chainid(ichain),ires,(coordat(k,iatom),k=1,3)!,vtot(i)
                  iatom=iatom+1
               enddo
!-------------------------------------------------------------------------------
            else
               write (iunit,60) iatom,restyp(iti,molnum(i)),chainid(ichain),&
                                        ires,(c(j,i),j=1,3),vtot(i)
            endif
            if ((iti.ne.10).and.(molnum(i).ne.5)) then
!               iatom=iatom+1
               if (molnum(i).eq.1) then
                  write (iunit,20) iatom,restyp(iti,1),chainid(ichain),&
                              ires,(c(j,nres+i),j=1,3),vtot(i+nres)
!               else if (molnum(i).eq.2) then
!                  if (istype(i).eq.0) istype(i)=1
!                  write (iunit,50) iatom,sugartyp(istype(i)),restyp(iti,2), &
!                   chainid(ichain),ires,(c(j,nres+i),j=1,3),vtot(i+nres)
               endif
            endif
         endif
      enddo
      print *, iatom,natom
      write (iunit,'(a)') 'TER'
!      do i=nnt,nct-1
!        if (itype(i,1).eq.ntyp1) cycle
!        if ((itype(i,1).eq.10 .and. itype(i+1,1).ne.ntyp1).or.(molnum(i).eq.5)) then
!          write (iunit,30) ica(i),ica(i+1)
!        else if (itype(i,1).ne.10 .and. itype(i+1,1).ne.ntyp1) then
!          write (iunit,30) ica(i),ica(i+1),ica(i)+1
!        else if (itype(i,1).ne.10 .and. itype(i+1,1).eq.ntyp1) then
!          write (iunit,30) ica(i),ica(i)+1
!        endif
!      enddo
!      if ((itype(nct,1).ne.10).and.(molnum(i).ne.5)) then
!        write (iunit,30) ica(nct),ica(nct)+1
!      endif
!      do i=1,nss
!       if (dyn_ss) then
!        write (iunit,30) ica(idssb(i))+1,ica(jdssb(i))+1
!       else
!        write (iunit,30) ica(ihpb(i)-nres)+1,ica(jhpb(i)-nres)+1
!       endif
!      enddo
      write (iunit,'(a6)') 'ENDMDL'     
  10  FORMAT ('ATOM',I7,'  CA  ',A3,1X,A1,I4,4X,3F8.3,f15.3)

  20  FORMAT ('ATOM',I7,'  CB  ',A3,1X,A1,I4,4X,3F8.3,f15.3)
  40  FORMAT ("ATOM",I7,1X,A5,1X,2A1,1X,A1,I4,4X,3F8.3,f15.3)
  50  FORMAT ("ATOM",I7,"  C1' ",1X,2A1,1X,A1,I4,4X,3F8.3,f15.3)

  30  FORMAT ('CONECT',8I5)
  60  FORMAT ('HETATM',I5,'  CA  ',A3,1X,A1,I4,4X,3F8.3,f15.3)

      return
      end subroutine pdbout
!-----------------------------------------------------------------------------
      subroutine MOL2out(etot,tytul)
! Prints the Cartesian coordinates of the alpha-carbons in the Tripos mol2 
! format.
      use geometry_data, only: c
      use energy_data
   !  use control
!      implicit real*8 (a-h,o-z)
!      include 'DIMENSIONS'
!      include 'COMMON.CHAIN'
!      include 'COMMON.INTERACT'
!      include 'COMMON.NAMES'
!      include 'COMMON.IOUNITS'
!      include 'COMMON.HEADER'
!      include 'COMMON.SBRIDGE'
      character(len=32) :: tytul,fd
      character(len=3) :: zahl
      character(len=6) :: res_num,pom	!,ucase

!el  local variables
      integer :: i,j
      real(kind=8) :: etot

#ifdef AIX
      call fdate_(fd)
#elif (defined CRAY)
      call date(fd)
#else
      call fdate(fd)
#endif
      write (imol2,'(a)') '#'
      write (imol2,'(a)') &
       '#         Creating user name:           unres'
      write (imol2,'(2a)') '#         Creation time:                ',&
       fd
      write (imol2,'(/a)') '\@<TRIPOS>MOLECULE'
      write (imol2,'(a)') tytul
      write (imol2,'(5i5)') nct-nnt+1,nct-nnt+nss+1,nct-nnt+nss+1,0,0
      write (imol2,'(a)') 'SMALL'
      write (imol2,'(a)') 'USER_CHARGES'
      write (imol2,'(a)') '\@<TRIPOS>ATOM' 
      do i=nnt,nct
        write (zahl,'(i3)') i
        pom=ucase(restyp(itype(i,1),1))
        res_num = pom(:3)//zahl(2:)
        write (imol2,10) i,(c(j,i),j=1,3),i,res_num,0.0
      enddo
      write (imol2,'(a)') '\@<TRIPOS>BOND'
      do i=nnt,nct-1
        write (imol2,'(i5,2i6,i2)') i-nnt+1,i-nnt+1,i-nnt+2,1
      enddo
      do i=1,nss
        write (imol2,'(i5,2i6,i2)') nct-nnt+i,ihpb(i),jhpb(i),1
      enddo
      write (imol2,'(a)') '\@<TRIPOS>SUBSTRUCTURE'
      do i=nnt,nct
        write (zahl,'(i3)') i
        pom = ucase(restyp(itype(i,1),1))
        res_num = pom(:3)//zahl(2:)
        write (imol2,30) i-nnt+1,res_num,i-nnt+1,0
      enddo
  10  FORMAT (I7,' CA      ',3F10.4,' C.3',I8,1X,A,F11.4,' ****')
  30  FORMAT (I7,1x,A,I14,' RESIDUE',I13,' ****  ****')
      return
      end subroutine MOL2out
!-----------------------------------------------------------------------------
      subroutine intout

      use geometry_data
      use energy_data, only: itype
   !  use control
!      implicit real*8 (a-h,o-z)
!      include 'DIMENSIONS'
!      include 'COMMON.IOUNITS'
!      include 'COMMON.CHAIN'
!      include 'COMMON.VAR'
!      include 'COMMON.LOCAL'
!      include 'COMMON.INTERACT'
!      include 'COMMON.NAMES'
!      include 'COMMON.GEO'
!      include 'COMMON.TORSION'
!el  local variables
      integer :: i,iti

      write (iout,'(/a)') 'Geometry of the virtual chain.'
      write (iout,'(7a)') '  Res  ','         d','     Theta',&
       '       Phi','       Dsc','     Alpha','      Omega'
      do i=1,nres
	iti=itype(i,1)
!         print *,vbld(i),"vbld(i)",i
        write (iout,'(a3,i4,6f10.3)') restyp(iti,1),i,vbld(i),&
           rad2deg*theta(i),rad2deg*phi(i),vbld(nres+i),rad2deg*alph(i),&
           rad2deg*omeg(i)
      enddo
      return
      end subroutine intout
!-----------------------------------------------------------------------------
#ifdef CLUSTER
      subroutine briefout(it,ener,free)!,plik)
#else
      subroutine briefout(it,ener)
#endif
      use geometry_data
      use energy_data
   !  use control
!      implicit real*8 (a-h,o-z)
!      include 'DIMENSIONS'
!      include 'COMMON.IOUNITS'
!      include 'COMMON.CHAIN'
!      include 'COMMON.VAR'
!      include 'COMMON.LOCAL'
!      include 'COMMON.INTERACT'
!      include 'COMMON.NAMES'
!      include 'COMMON.GEO'
!      include 'COMMON.SBRIDGE'
!     print '(a,i5)',intname,igeom
!el  local variables
      integer :: i,it
      real(kind=8) :: ener,free
!     character(len=80) :: plik
!      integer :: iii

#if defined(AIX) || defined(PGI)
      open (igeom,file=intname,position='append')
#else
      open (igeom,file=intname,access='append')
#endif
#ifdef WHAM_RUN
!      iii=igeom
      igeom=iout
#endif
      print *,nss
      IF (NSS.LE.9) THEN
#ifdef CLUSTER
        WRITE (igeom,180) IT,ENER,free,NSS,(IHPB(I),JHPB(I),I=1,NSS)
      ELSE
        WRITE (igeom,180) IT,ENER,free,NSS,(IHPB(I),JHPB(I),I=1,8)
#else
        WRITE (igeom,180) IT,ENER,NSS,(IHPB(I),JHPB(I),I=1,NSS)
      ELSE
        WRITE (igeom,180) IT,ENER,NSS,(IHPB(I),JHPB(I),I=1,9)
#endif
        WRITE (igeom,190) (IHPB(I),JHPB(I),I=10,NSS)
      ENDIF
!     IF (nvar.gt.nphi) WRITE (igeom,200) (RAD2DEG*THETA(I),I=3,NRES)
      WRITE (igeom,200) (RAD2DEG*THETA(I),I=3,NRES)
      WRITE (igeom,200) (RAD2DEG*PHI(I),I=4,NRES)
!     if (nvar.gt.nphi+ntheta) then
        write (igeom,200) (rad2deg*alph(i),i=2,nres-1)
        write (igeom,200) (rad2deg*omeg(i),i=2,nres-1)
!     endif
      close(igeom)
  180 format (I5,F12.3,I2,9(1X,2I3))
  190 format (3X,11(1X,2I3))
  200 format (8F10.4)
      return
      end subroutine briefout
!-----------------------------------------------------------------------------
#ifdef WINIFL
      subroutine fdate(fd)
      character(len=32) :: fd
      write(fd,'(32x)')
      return
      end subroutine fdate
#endif
!-----------------------------------------------------------------------------
#ifdef WHAM_RUN
      real(kind=8) function gyrate(jcon)
#else
      real(kind=8) function gyrate()
#endif

      use geometry_data, only: c
   !  use geometry
      use energy_data
!      implicit real*8 (a-h,o-z)
!      include 'DIMENSIONS'
!      include 'COMMON.INTERACT'
!      include 'COMMON.CHAIN'
      real(kind=8) :: rg
      real(kind=8),dimension(3) :: cen
!el  local variables
      integer :: i,j,jcon

      do j=1,3
       cen(j)=0.0d0
      enddo

      do i=nnt,nct
          do j=1,3
            cen(j)=cen(j)+c(j,i)
          enddo
      enddo
      do j=1,3
            cen(j)=cen(j)/dble(nct-nnt+1)
      enddo
      rg = 0.0d0
      do i = nnt, nct
        do j=1,3
         rg = rg + (c(j,i)-cen(j))**2 
        enddo
      end do
#ifdef WHAM_RUN
      gyrate = dsqrt(rg/dble(nct-nnt+1))
#else
      gyrate = sqrt(rg/dble(nct-nnt+1))
#endif
      return
      end function gyrate
#ifdef WHAM_RUN
!-----------------------------------------------------------------------------
! readrtns.F WHAM
      subroutine reads(rekord,lancuch,wartosc,default)
!      implicit none
      character*(*) :: rekord,lancuch,wartosc,default
      character(len=80) :: aux
      integer :: lenlan,lenrec,iread,ireade
!el      external ilen
!el      logical iblnk
!el      external iblnk
      lenlan=ilen(lancuch)
      lenrec=ilen(rekord)
      iread=index(rekord,lancuch(:lenlan)//"=")
!      print *,"rekord",rekord," lancuch",lancuch
!      print *,"iread",iread," lenlan",lenlan," lenrec",lenrec
      if (iread.eq.0) then
        wartosc=default
        return
      endif
      iread=iread+lenlan+1
!      print *,"iread",iread
!      print *,"|",rekord(iread:iread),"|",iblnk(rekord(iread:iread))
      do while (iread.le.lenrec .and. iblnk(rekord(iread:iread)))
        iread=iread+1
!      print *,"|",rekord(iread:iread),"|",iblnk(rekord(iread:iread))
      enddo
!      print *,"iread",iread
      if (iread.gt.lenrec) then
         wartosc=default
        return
      endif
      ireade=iread+1
!      print *,"ireade",ireade
      do while (ireade.lt.lenrec .and. &
         .not.iblnk(rekord(ireade:ireade)))
        ireade=ireade+1
      enddo
      wartosc=rekord(iread:ireade)
      return
      end subroutine reads
#endif
!-----------------------------------------------------------------------------
! permut.F
!-----------------------------------------------------------------------------
      subroutine permut(isym)

      use geometry_data, only: tabperm
!      implicit real*8 (a-h,o-z) 
!      include 'DIMENSIONS'
!      include 'COMMON.LOCAL'
!      include 'COMMON.VAR'
!      include 'COMMON.CHAIN'
!      include 'COMMON.INTERACT'
!      include 'COMMON.IOUNITS'
!      include 'COMMON.GEO'
!      include 'COMMON.NAMES'
!      include 'COMMON.CONTROL'

      integer :: n,isym
!      logical nextp
!el      external nextp
      integer,dimension(isym) :: a
!      parameter(n=symetr)
!el local variables
      integer :: kkk,i

      n=isym
      if (n.eq.1) then
        tabperm(1,1)=1
        return
      endif
      kkk=0
      do i=1,n
      a(i)=i
      enddo
   10 print *,(a(i),i=1,n)
      kkk=kkk+1
      do i=1,n
      tabperm(kkk,i)=a(i)
!      write (iout,*) "tututu", kkk
      enddo
      if(nextp(n,a)) go to 10
      return
      end subroutine permut
!-----------------------------------------------------------------------------
      logical function nextp(n,a)

      integer :: n,i,j,k,t
!      logical :: nextp
      integer,dimension(n) :: a
      i=n-1
   10 if(a(i).lt.a(i+1)) go to 20
      i=i-1
      if(i.eq.0) go to 20
      go to 10
   20 j=i+1
      k=n
   30 t=a(j)
      a(j)=a(k)
      a(k)=t
      j=j+1
      k=k-1
      if(j.lt.k) go to 30
      j=i
      if(j.ne.0) go to 40
      nextp=.false.
      return
   40 j=j+1
      if(a(j).lt.a(i)) go to 40
      t=a(i)
      a(i)=a(j)
      a(j)=t
      nextp=.true.
      return
      end function nextp
!-----------------------------------------------------------------------------
!-----------------------------------------------------------------------------
! readpdb.F
!-----------------------------------------------------------------------------
      subroutine readpdb
! Read the PDB file and convert the peptide geometry into virtual-chain 
! geometry.
      use geometry_data
      use energy_data
      use control_data
      use compare_data
      use MPI_data
      use geometry
      integer :: i,j,ibeg,ishift1,ires,iii,ires_old,ishift,k
      logical :: lprn=.true.,fail
      real(kind=8),dimension(3) :: e1,e2,e3
      real(kind=8) :: dcj,efree_temp
      character(len=3) :: seq,res
      character(len=5) :: atom
      character(len=80) :: card
      real(kind=8),dimension(3,20) :: sccor
      integer :: kkk,lll,icha,kupa,molecule,counter,seqalingbegin
      integer :: isugar,molecprev,firstion
      character*1 :: sugar
      real(kind=8) :: cou
      real(kind=8),dimension(3) :: ccc
      integer,dimension(2,maxres/3) :: hfrag_alloc
      integer,dimension(4,maxres/3) :: bfrag_alloc
      real(kind=8),dimension(3,maxres2+2,maxperm) :: cref_alloc
      real(kind=8),dimension(:,:), allocatable  :: c_temporary
      integer,dimension(:,:) , allocatable  :: itype_temporary
      integer,dimension(:),allocatable :: istype_temp
      efree_temp=0.0d0
      ibeg=1
      ishift1=0
      ishift=0
      molecule=1
      counter=0
      ires=0
      ires_old=0
#ifdef WHAM_RUN
      do i=1,nres
         do j=1,5
            itype(i,j)=0
         enddo
      enddo
#endif
      nres=0
      iii=0
      lsecondary=.false.
      nhfrag=0
      nbfrag=0
      do j=1,5
         nres_molec(j)=0
      enddo
      
       
!-----------------------------
      allocate(hfrag(2,maxres/3))
      allocate(bfrag(4,maxres/3))
      if(.not. allocated(istype)) allocate(istype(maxres))
      do i=1,100000
         read (ipdbin,'(a80)',end=10) card
         if (card(:5).eq.'HELIX') then
            nhfrag=nhfrag+1
            lsecondary=.true.
            read(card(22:25),*) hfrag(1,nhfrag)
            read(card(34:37),*) hfrag(2,nhfrag)
         endif
         if (card(:5).eq.'SHEET') then
            nbfrag=nbfrag+1
            lsecondary=.true.
            read(card(24:26),*) bfrag(1,nbfrag)
            read(card(35:37),*) bfrag(2,nbfrag)
!rc----------------------------------------
!rc  to be corrected !!!
            bfrag(3,nbfrag)=bfrag(1,nbfrag)
            bfrag(4,nbfrag)=bfrag(2,nbfrag)
!rc----------------------------------------
         endif
         if (card(:3).eq.'END') then
            goto 10
         else if (card(:3).eq.'TER') then
! End current chain
            ires_old=ires+2
            ishift=ishift+1
            ishift1=ishift1+1
            itype(ires_old,molecule)=ntyp1_molec(molecule)
            itype(ires_old-1,molecule)=ntyp1_molec(molecule)
            nres_molec(molecule)=nres_molec(molecule)+2
            ibeg=2
            if (unres_pdb) then
               do j=1,3
                  dc(j,ires)=sccor(j,iii)
               enddo
            else
               call sccenter(ires,iii,sccor)
            endif
            iii=0
         endif
! Read free energy
         if (index(card,"FREE ENERGY").gt.0) read(card(35:),*) efree_temp
! Fish out the ATOM cards.
         if (index(card(1:4),'ATOM').gt.0) then  
            read (card(12:16),*) atom
            read (card(23:26),*) ires
            read (card(18:20),'(a3)') res
            if (ires-ishift+ishift1.ne.ires_old) then
               if (ibeg.eq.0) then
                  if (unres_pdb) then
                     do j=1,3
                        dc(j,ires+ishift1-ishift-1)=sccor(j,iii)
                     enddo
                  else
                     call sccenter(ires_old,iii,sccor)
                  endif
                  iii=0
               endif
! Start new residue.
               if (res.eq.'Cl-' .or. res.eq.'Na+') then
                  ires=ires_old
                  cycle
               else if (ibeg.eq.1) then
                  write (iout,*) "BEG ires",ires
                  ishift=ires-1
                  if (res.ne.'GLY' .and. res.ne. 'ACE') then
                     ishift=ishift-1
                     itype(1,1)=ntyp1
                     nres_molec(molecule)=nres_molec(molecule)+1
                  endif
                  ires=ires-ishift+ishift1
                  ires_old=ires
                  ibeg=0 
               else if (ibeg.eq.2) then
! Start a new chain
                  ishift=-ires_old+ires-1 !!!!!
                  ishift1=ishift1-1    !!!!!
                  ires=ires-ishift+ishift1
                  ires_old=ires
                  ibeg=0
               else
                  ishift=ishift-(ires-ishift+ishift1-ires_old-1)
                  ires=ires-ishift+ishift1
                  ires_old=ires
               endif 
               if (res.eq.'ACE' .or. res.eq.'NHE') then
                  itype(ires,1)=10
               else
                  if (atom.eq.'CA  '.or.atom.eq.'N   ') then
                     molecule=1
                     itype(ires,molecule)=rescode(ires,res,0,molecule)
                     firstion=0
                  else
                     molecule=2
                     itype(ires,molecule)=rescode(ires,res(2:3),0,molecule)
                     read (card(19:19),'(a1)') sugar
                     isugar=sugarcode(sugar,ires)
                     istype(ires)=isugar

                  endif
               endif
            else
               ires=ires-ishift+ishift1
            endif
!            if (card(27:27).eq."A" .or. card(27:27).eq."B") then
!            endif
            if (atom.eq.'CA'.or. atom.eq.'CH3'.or. &
          res.eq.'NHE'.and.atom(:2).eq.'HN') then
               read(card(31:54),'(3f8.3)') (c(j,ires),j=1,3)
#ifdef DEBUG
               write (iout,'(2i3,2x,a,3f8.3)') &
          ires,itype(ires,1),res,(c(j,ires),j=1,3)
#endif
               iii=iii+1
               nres_molec(molecule)=nres_molec(molecule)+1
               do j=1,3
                  sccor(j,iii)=c(j,ires)
               enddo
            else if (.not.unres_pdb.and.(atom.eq."C1'".or.atom.eq."C2'".or.atom.eq."C3'".or.atom.eq."C4'".or.atom.eq."O4'")) then
               read(card(31:54),'(3f8.3)') (ccc(j),j=1,3)
               counter=counter+1
               do j=1,3
                  c(j,ires)=c(j,ires)+ccc(j)/5.0
               enddo
               print *,counter,molecule
               if (counter.eq.5) then
                  nres_molec(molecule)=nres_molec(molecule)+1
                  firstion=0
                  counter=0
               endif
            else if (atom.eq."C5'") then
               read (card(19:19),'(a1)') sugar
               isugar=sugarcode(sugar,ires)
               if (ibeg.eq.1) then
                  istype(1)=isugar
               else
                  istype(ires)=isugar
               endif
               if (unres_pdb) then
                  molecule=2
                  read(card(31:54),'(3f8.3)') (c(j,ires),j=1,3)
                  nres_molec(molecule)=nres_molec(molecule)+1
                  print *,"nres_molec(molecule)",nres_molec(molecule),ires
               else
                  iii=iii+1
                  read(card(31:54),'(3f8.3)') (sccor(j,iii),j=1,3)
               endif
            else if ((atom.eq."C1'").and.unres_pdb) then
               iii=iii+1
               read(card(31:54),'(3f8.3)') (sccor(j,iii),j=1,3)
            else if (atom.ne.'O'.and.atom(1:1).ne.'H'.and.atom.ne.'N' &
      .and.atom.ne.'C'.and.atom(:2).ne.'1H'.and.atom(:2).ne.'2H'.and. &
      atom.ne.'OXT'.and.atom(:2).ne.'3H'.and.atom.ne.'P  '.and.atom(1:1) &
      .ne.'H'.and.atom.ne.'OP1'.and. atom.ne.'OP2 '.and.atom.ne.'OP3') then
               if ((molecule.ne.2).or.(atom(3:3).ne."'")) then
                  iii=iii+1
                  read(card(31:54),'(3f8.3)') (sccor(j,iii),j=1,3)
               endif
            endif
         else if ((ions).and.(card(1:6).eq.'HETATM')) then
            if (firstion.eq.0) then 
               firstion=1
               if (unres_pdb) then
                  do j=1,3
                     dc(j,ires)=sccor(j,iii)
                  enddo
               else
                  call sccenter(ires,iii,sccor)
               endif
            endif
            read (card(12:16),*) atom
            print *,"HETATOM", atom
            read (card(18:20),'(a3)') res
            if ((atom(1:2).eq.'NA').or.(atom(1:2).eq.'CL').or. &
      (atom(1:2).eq.'CA').or.(atom(1:2).eq.'MG').or.(atom(1:2).eq.'K ')) then
            ires=ires+1
            if (molecule.ne.5) molecprev=molecule
               molecule=5
               nres_molec(molecule)=nres_molec(molecule)+1
               print *,"HERE",nres_molec(molecule)
               res=res(2:3)//' '
               itype(ires,molecule)=rescode(ires,res,0,molecule)
               read(card(31:54),'(3f8.3)') (c(j,ires),j=1,3)
            endif
         endif !atom
      enddo
   10 write (iout,'(a,i5)') ' Number of residues found: ',ires
      if (ires.eq.0) return
! Calculate dummy residue coordinates inside the "chain" of a multichain
! system
      nres=ires
      if ((ires_old.ne.ires).and.(molecule.ne.5)) &
       nres_molec(molecule)=nres_molec(molecule)-2
      print *,'I have',nres, nres_molec(:)
      do k=1,4 ! ions are without dummy 
         if (nres_molec(k).eq.0) cycle
         do i=2,nres-1
            if (itype(i,k).eq.ntyp1_molec(k)) then
               if (itype(i+1,k).eq.ntyp1_molec(k)) then
                  if (itype(i+2,k).eq.0) then 
                     do j=1,5
                        if (itype(i+2,j).ne.0) then
                           itype(i+1,k)=0
                           itype(i+1,j)=ntyp1_molec(j)
                           nres_molec(k)=nres_molec(k)-1
                           nres_molec(j)=nres_molec(j)+1
                           go to 3331
                        endif
                     enddo
 3331                continue
                  endif
! 16/01/2014 by Adasko: Adding to dummy atoms in the chain
! first is connected prevous chain (itype(i+1,1).eq.ntyp1)=true
! second dummy atom is conected to next chain itype(i+1,1).eq.ntyp1=false
!           if (unres_pdb) then
! 2/15/2013 by Adam: corrected insertion of the last dummy residue
                  do j=1,3
                     dcj=(c(j,i-2)-c(j,i-3))/2.0
                     if (dcj.eq.0) dcj=1.23591524223
                     c(j,i)=c(j,i-1)+dcj
                     c(j,nres+i)=c(j,i)
                  enddo
               else
! 2/15/2013 by Adam: corrected insertion of the first dummy residue
                  do j=1,3
                     c(j,i)=c(j,i+1)-1.9d0*e2(j)
                  enddo
                  do j=1,3
                     dcj=(c(j,i+3)-c(j,i+2))/2.0
                     if (dcj.eq.0) dcj=1.23591524223
                     c(j,i)=c(j,i+1)-dcj
                     c(j,nres+i)=c(j,i)
                  enddo
               endif !itype(i+1,1).eq.ntyp1
            endif  !itype.eq.ntyp1
         enddo
      enddo
! Calculate the CM of the last side chain.
      if (iii.gt.0)  then
         if (unres_pdb) then
            do j=1,3
               dc(j,ires)=sccor(j,iii)
            enddo
         else
            call sccenter(ires,iii,sccor)
         endif
      endif
      nsup=nres
      nstart_sup=1
      if ((itype(nres,1).ne.10)) then
         nres=nres+1
         if (molecule.eq.5) molecule=molecprev
         itype(nres,molecule)=ntyp1_molec(molecule)
         nres_molec(molecule)=nres_molec(molecule)+1
! 2/15/2013 by Adam: corrected insertion of the last dummy residue
         do j=1,3
            dcj=(c(j,nres-2)-c(j,nres-3))/2.0
            c(j,nres)=c(j,nres-1)+dcj
            c(j,2*nres)=c(j,nres)
         enddo
      endif

!el kontrola nres w pliku inputowym WHAM-a w porownaniu z wartoscia wczytana z pliku pdb
#ifdef WHAM_RUN
      if (nres.ne.nres0) then
         write (iout,*) "Error: wrong parameter value: NRES=",nres, &
                        " NRES0=",nres0
         stop "Error nres value in WHAM input"
      endif
#endif
      do i=2,nres-1
         do j=1,3
            c(j,i+nres)=dc(j,i)
         enddo
      enddo
      do j=1,3
         c(j,nres+1)=c(j,1)
         c(j,2*nres)=c(j,nres)
      enddo
      if (itype(1,1).eq.ntyp1) then
         nsup=nsup-1
         nstart_sup=2
         if (unres_pdb) then
! 2/15/2013 by Adam: corrected insertion of the first dummy residue
            call refsys(2,3,4,e1,e2,e3,fail)
            if (fail) then
               e2(1)=0.0d0
               e2(2)=1.0d0
               e2(3)=0.0d0
            endif
            do j=1,3
               c(j,1)=c(j,2)-1.9d0*e2(j)
            enddo
         else
            do j=1,3
               dcj=(c(j,4)-c(j,3))/2.0
               c(j,1)=c(j,2)-dcj
               c(j,nres+1)=c(j,1)
            enddo
         endif
      endif
! First lets assign correct dummy to correct type of chain
! 1) First residue
      if (itype(1,1).eq.ntyp1) then
         if (itype(2,1).eq.0) then
            do j=2,5
               if (itype(2,j).ne.0) then
                  itype(1,1)=0
                  itype(1,j)=ntyp1_molec(j)
                  nres_molec(1)=nres_molec(1)-1
                  nres_molec(j)=nres_molec(j)+1
                  go to 3231
               endif
            enddo
3231        continue
         endif
      endif
      print *,'I have',nres, nres_molec(:)

! Copy the coordinates to reference coordinates
! Calculate internal coordinates.
      if (lprn) then
         write (iout,'(/a)') "Cartesian coordinates of the reference structure"
         write (iout,'(a,3(3x,a5),5x,3(3x,a5))') "Residue","X(CA)","Y(CA)", &
                "Z(CA)","X(SC)","Y(SC)","Z(SC)"
         do ires=1,nres
            write (iout,'(5(a3,1x),i3,3f8.3,5x,3f8.3)') &
      (restyp(itype(ires,j),j),j=1,5),ires,(c(j,ires),j=1,3),(c(j,ires+nres),j=1,3)
         enddo
      endif
! znamy już nres wiec mozna alokowac tablice
! Calculate internal coordinates.
      if(me.eq.king.or..not.out1file)then
         write (iout,'(a)')  &
      "Backbone and SC coordinates as read from the PDB"
         do ires=1,nres
            write (iout,'(2i3,2x,a,3f8.3,5x,3f8.3)') ires,itype(ires,1), &
        restyp(itype(ires,1),1),(c(j,ires),j=1,3),(c(j,nres+ires),j=1,3)
         enddo
      endif
! NOW LETS ROCK! SORTING
      allocate(c_temporary(3,2*nres))
      allocate(itype_temporary(nres,5))
      if (.not.allocated(molnum)) allocate(molnum(nres+1))
      if (.not.allocated(istype)) write(iout,*) &
                   "SOMETHING WRONG WITH ISTYTPE"
      allocate(istype_temp(nres))
      itype_temporary(:,:)=0
      seqalingbegin=1
      do k=1,5
         do i=1,nres
            if (itype(i,k).ne.0) then
               do j=1,3
                  c_temporary(j,seqalingbegin)=c(j,i)
                  c_temporary(j,seqalingbegin+nres)=c(j,i+nres)
               enddo
               itype_temporary(seqalingbegin,k)=itype(i,k)
               print *,i,k,itype(i,k),itype_temporary(seqalingbegin,k),seqalingbegin
               istype_temp(seqalingbegin)=istype(i)
               molnum(seqalingbegin)=k
               seqalingbegin=seqalingbegin+1
            endif
         enddo
      enddo
      do i=1,2*nres
         do j=1,3
            c(j,i)=c_temporary(j,i)
         enddo
      enddo
      do k=1,5
         do i=1,nres
            itype(i,k)=itype_temporary(i,k)
            istype(i)=istype_temp(i)
         enddo
      enddo
! 2/15/2013 by Adam: corrected insertion of the first dummy residue
      if (lprn) then
         write (iout,'(/a)') &
      "Cartesian coordinates of the reference structure after sorting"
         write (iout,'(a,3(3x,a5),5x,3(3x,a5))') "Residue","X(CA)", &
                           "Y(CA)","Z(CA)","X(SC)","Y(SC)","Z(SC)"
         do ires=1,nres
            write (iout,'(5(a3,1x),i3,3f8.3,5x,3f8.3)') &
      (restyp(itype(ires,j),j),j=1,5),ires,(c(j,ires),j=1,3),(c(j,ires+nres),j=1,3)
         enddo
      endif
      if(.not.allocated(vbld)) then
         allocate(vbld(2*nres))
         do i=1,2*nres
            vbld(i)=0.d0
         enddo
      endif
      if(.not.allocated(vbld_inv)) then
         allocate(vbld_inv(2*nres))
         do i=1,2*nres
            vbld_inv(i)=0.d0
         enddo
      endif
      if(.not.allocated(theta)) then
         allocate(theta(nres+2))
         theta(:)=0.0d0
      endif
      if(.not.allocated(phi)) allocate(phi(nres+2))
      if(.not.allocated(alph)) allocate(alph(nres+2))
      if(.not.allocated(omeg)) allocate(omeg(nres+2))
      if(.not.allocated(thetaref)) allocate(thetaref(nres+2))
      if(.not.allocated(phiref)) allocate(phiref(nres+2))
      if(.not.allocated(costtab)) allocate(costtab(nres))
      if(.not.allocated(sinttab)) allocate(sinttab(nres))
      if(.not.allocated(cost2tab)) allocate(cost2tab(nres))
      if(.not.allocated(sint2tab)) allocate(sint2tab(nres))
      if(.not.allocated(xxref)) allocate(xxref(nres))
      if(.not.allocated(yyref)) allocate(yyref(nres))
      if(.not.allocated(zzref)) allocate(zzref(nres))
      if(.not.allocated(dc_norm)) then
         allocate(dc_norm(3,0:2*nres+2))
         dc_norm(:,:)=0.d0
      endif
      call int_from_cart(.true.,.false.)
      call sc_loc_geom(.false.)
      do i=1,nres
         thetaref(i)=theta(i)
         phiref(i)=phi(i)
      enddo
      do i=1,nres-1
         do j=1,3
            dc(j,i)=c(j,i+1)-c(j,i)
            dc_norm(j,i)=dc(j,i)*vbld_inv(i+1)
         enddo
      enddo
      do i=2,nres-1
         do j=1,3
            dc(j,i+nres)=c(j,i+nres)-c(j,i)
            dc_norm(j,i+nres)=dc(j,i+nres)*vbld_inv(i+nres)
         enddo
      enddo
      if(.not.allocated(cref)) allocate(cref(3,2*nres+2,maxperm))
      if(.not.allocated(chain_rep)) allocate(chain_rep(3,2*nres+2,maxsym))
      if(.not.allocated(tabperm)) allocate(tabperm(maxperm,maxsym))
!-----------------------------
      kkk=1
      lll=0
      cou=1
      write (iout,*) "symetr", symetr
      do i=1,nres
         lll=lll+1
         if (i.gt.1) then
            if ((itype(i-1,1).eq.ntyp1).and.(i.gt.2)) then
               chain_length=lll-1
               kkk=kkk+1
               lll=1
            endif
         endif
         do j=1,3
            cref(j,i,cou)=c(j,i)
            cref(j,i+nres,cou)=c(j,i+nres)
            if (i.le.nres) then
               chain_rep(j,lll,kkk)=c(j,i)
               chain_rep(j,lll+nres,kkk)=c(j,i+nres)
            endif
         enddo
      enddo
      write (iout,*) chain_length
      if (chain_length.eq.0) chain_length=nres
      do j=1,3
!         chain_rep(j,chain_length,symetr)=chain_rep(j,chain_length,1)
!         chain_rep(j,chain_length+nres,symetr)=chain_rep(j,chain_length+nres,1)
      enddo
! makes copy of chains
      write (iout,*) "symetr", symetr
      do j=1,3
         dc(j,0)=c(j,1)
      enddo
      if (symetr.gt.1) then
         call permut(symetr)
         nperm=1
         do i=1,symetr
            nperm=nperm*i
         enddo
         do i=1,nperm
            write(iout,*) (tabperm(i,kkk),kkk=1,4)
         enddo
         do i=1,nperm
            cou=0
            do kkk=1,symetr
               icha=tabperm(i,kkk)
               do lll=1,chain_length
                  cou=cou+1
                  if (cou.le.nres) then
                     do j=1,3
                        kupa=mod(lll,chain_length)
                        iprzes=(kkk-1)*chain_length+lll
                        if (kupa.eq.0) kupa=chain_length
                           cref(j,iprzes,i)=chain_rep(j,kupa,icha)
                           cref(j,iprzes+nres,i)=chain_rep(j,kupa+nres,icha)
                     enddo
                  endif
               enddo
            enddo
         enddo
      endif
!-koniec robienia kopii
! diag
      do kkk=1,nperm
         write (iout,*) "nowa struktura", nperm
         do i=1,nres
            write (iout,110) restyp(itype(i,1),1),i,cref(1,i,kkk),cref(2,i,kkk), &
      cref(3,i,kkk),cref(1,nres+i,kkk),cref(2,nres+i,kkk),cref(3,nres+i,kkk)
         enddo
  100 format (//'              alpha-carbon coordinates       ',&
                '     centroid coordinates'/ &
                '       ', 6X,'X',11X,'Y',11X,'Z', &
                                10X,'X',11X,'Y',11X,'Z')
  110 format (a,'(',i3,')',6f12.5)
      enddo
!c enddiag
      do j=1,nbfrag     
         do i=1,4                                                       
            bfrag(i,j)=bfrag(i,j)-ishift
         enddo
      enddo
      do j=1,nhfrag
         do i=1,2
            hfrag(i,j)=hfrag(i,j)-ishift
         enddo
      enddo
      ishift_pdb=ishift
      return
      end subroutine readpdb
!-----------------------------------------------------------------------------
!-----------------------------------------------------------------------------
      subroutine readres(path)
      use geometry_data
!      use energy_data, only: itype,istype
!      use control_data
!      use compare_data
!      use MPI_data
      use names
      character(len=26) :: path,fullname
      logical :: filex
      allocate(resnatom(2,5),resat(2,5,34),rescoord(2,5,3,34),resd(2,5), &
               reso3(2,5,3),resc5(2,5,3))
      path=trim(path)//"data/"
      do i=1,5
         fullname=trim(path)//"D"//restyp(i,2)(1:1)//".pdb"
         print *, fullname
         inquire(file=fullname,exist=filex)
         if (filex) then
            print *, sugartyp(1),restyp(i,2),"file exists"
            open(666,file=fullname)
            read(666,*) resd(1,i)
            j=0
            do
               read(666,'(12x,a5,13x,3f8.3)',end=1337) resat(1,i,j+1),(rescoord(1,i,k,j+1),k=1,3)
               j=j+1
               cycle
1337           exit
            enddo
            resnatom(1,i)=j
            close(666)
            do k=1,j
               if (index(resat(1,i,k),"O3'").ne.0) then 
                  do l=1,3
                     reso3(1,i,l)=rescoord(1,i,l,k)
                  enddo
               endif
            enddo   
            do k=1,j
               if (index(resat(1,i,k),"C5'").ne.0) then 
                  do l=1,3
                     resc5(1,i,l)=rescoord(1,i,l,k)
                  enddo
               endif
            enddo   
         else
            print *, sugartyp(1),restyp(i,2),"file does not exist"
            resnatom(1,i)=0
         endif
         fullname=trim(path)//restyp(i,2)(1:1)//".pdb"
         print *, fullname
         inquire(file=fullname,exist=filex)
         if (filex) then
            print *, sugartyp(2),restyp(i,2),"file exists"
            open(666,file=fullname)
            j=0
            read(666,*) resd(2,i)
            do
               read(666,'(12x,a5,13x,3f8.3)',end=1338) resat(2,i,j+1),(rescoord(2,i,k,j+1),k=1,3)
               j=j+1
               cycle
1338           exit
            enddo
            resnatom(2,i)=j
            close(666)
            do k=1,j
               if (index(resat(2,i,k),"O3'").ne.0) then 
                  do l=1,3
                     reso3(2,i,l)=rescoord(2,i,l,k)
                  enddo
               endif
            enddo   
            do k=1,j
               if (index(resat(2,i,k),"C5'").ne.0) then 
                  do l=1,3
                     resc5(2,i,l)=rescoord(2,i,l,k)
                  enddo
               endif
            enddo   
         else
            print *, sugartyp(2),restyp(i,2),"file does not exist"
            resnatom(2,i)=0
         endif
      enddo      
      print *, resnatom   
      end subroutine readres

!-----------------------------------------------------------------------------
! rescode.f
!-----------------------------------------------------------------------------
      integer function rescode(iseq,nam,itype,molecule)

!      use io_base, only: ucase
!      implicit real*8 (a-h,o-z)
!      include 'DIMENSIONS'
!      include 'COMMON.NAMES'
!      include 'COMMON.IOUNITS'
      character(len=*) :: nam	!,ucase
      integer :: iseq,itype,i
      integer :: molecule
      print *,molecule,nam
      if (molecule.eq.1) then 
      if (itype.eq.0) then

      do i=-ntyp1_molec(molecule),ntyp1_molec(molecule)
        if (ucase(nam).eq.restyp(i,molecule)) then
          rescode=i
          return
        endif
      enddo

      else

      do i=-ntyp1_molec(molecule),ntyp1_molec(molecule)
        if (nam(1:1).eq.onelet(i)) then
          rescode=i
          return  
        endif  
      enddo

      endif
      else if (molecule.eq.2) then
      do i=1,ntyp1_molec(molecule)
         print *,nam(1:1),restyp(i,molecule)(1:1) 
        if (nam(2:2).eq.restyp(i,molecule)(1:1)) then
          rescode=i
          return
        endif
      enddo
      else if (molecule.eq.3) then
       write(iout,*) "SUGAR not yet implemented"
       stop
      else if (molecule.eq.4) then
       write(iout,*) "Explicit LIPID not yet implemented"
       stop
      else if (molecule.eq.5) then
      do i=1,ntyp1_molec(molecule)
        print *,i,restyp(i,molecule)(1:2)
        if (ucase(nam(1:2)).eq.restyp(i,molecule)(1:2)) then
          rescode=i
          return
        endif
      enddo
      else   
       write(iout,*) "molecule not defined"
      endif
      write (iout,10) iseq,nam
      stop
   10 format ('**** Error - residue',i4,' has an unresolved name ',a3)
      end function rescode
      integer function sugarcode(sugar,ires)
      character sugar
      integer ires
      if (sugar.eq.'D') then
        sugarcode=1
      else if (sugar.eq.' ') then
        sugarcode=2
      else
        write (iout,*) 'UNKNOWN sugar type for residue',ires,' ',sugar
        stop
      endif
      return
      end function sugarcode
!---------------------------------------------------------------------------
      end module io_base
